@@ -5,12 +5,14 @@
 use crate::photo::model::PictureId;
 
 use crate::machine_learning::face_extractor;
+use crate::path_encoding;
 use crate::people::FaceId;
+use crate::people::FaceToMigrate;
+use crate::people::MigratedFace;
 use crate::people::PersonId;
 use crate::people::model;
 use crate::people::model::PersonForRecognition;
 use crate::people::model::Rect;
-use crate::photo::model::Orientation;
 
 use anyhow::*;
 use rusqlite;
@@ -19,12 +21,16 @@ use rusqlite::params;
 use std::path::{Path, PathBuf};
 use std::result::Result::Ok;
 use std::sync::{Arc, Mutex};
+use tracing::warn;
 
 /// Repository of people data.
 /// Repository is backed by a Sqlite database.
 #[derive(Debug, Clone)]
 pub struct Repository {
-    /// Base path for photo thumbnails
+    /// Cache direcctory
+    cache_dir_base_path: PathBuf,
+
+    /// Data directory
     data_dir_base_path: PathBuf,
 
     /// Connection to backing Sqlite database.
@@ -34,12 +40,15 @@ pub struct Repository {
 impl Repository {
     /// Builds a Repository and creates operational tables.
     pub fn open(
+        cache_dir_base_path: &Path,
         data_dir_base_path: &Path,
         con: Arc<Mutex<rusqlite::Connection>>,
     ) -> Result<Repository> {
+        let cache_dir_base_path = PathBuf::from(cache_dir_base_path);
         let data_dir_base_path = PathBuf::from(data_dir_base_path);
 
         let repo = Repository {
+            cache_dir_base_path,
             data_dir_base_path,
             con,
         };
@@ -75,12 +84,12 @@ impl Repository {
                 faces.thumbnail_path AS face_thumbnail_path,
                 people.person_id AS person_id,
                 people.name AS person_name,
-                people.thumbnail_path AS person_thumbnail_path,
-                pictures.orientation
+                person_face.thumbnail_path AS person_thumbnail_path
             FROM pictures_faces AS faces
-            INNER JOIN pictures USING (picture_id)
             LEFT OUTER JOIN people USING (person_id)
-            WHERE picture_id = ?1 AND faces.is_ignored = FALSE
+            LEFT OUTER JOIN pictures_faces AS person_face
+                ON (person_face.person_id = faces.person_id AND person_face.is_thumbnail = TRUE)
+            WHERE faces.picture_id = ?1 AND faces.is_ignored = FALSE
             ORDER BY faces.nose_x ASC, faces.nose_y ASC",
         )?;
 
@@ -132,10 +141,12 @@ impl Repository {
         let con = self.con.lock().unwrap();
         let mut stmt = con.prepare(
             "SELECT
-                people.person_id AS person_id,
-                people.name AS person_name,
-                people.thumbnail_path AS person_thumbnail_path
-            FROM  people
+                p.person_id AS person_id,
+                p.name AS person_name,
+                f.thumbnail_path AS person_thumbnail_path
+            FROM people AS p
+            LEFT OUTER JOIN pictures_faces AS f
+                ON (f.person_id = p.person_id AND f.is_thumbnail = TRUE)
             WHERE person_id = ?1",
         )?;
 
@@ -155,7 +166,8 @@ impl Repository {
             let mut stmt = tx.prepare_cached(
                 "UPDATE pictures_faces
                 SET
-                    is_confirmed = FALSE
+                    is_confirmed = FALSE,
+                    is_thumbnail = FALSE
                 WHERE person_id = ?1",
             )?;
             stmt.execute(params![person_id.id(),])?;
@@ -190,10 +202,12 @@ impl Repository {
         let con = self.con.lock().unwrap();
         let mut stmt = con.prepare(
             "SELECT
-                people.person_id AS person_id,
-                people.name AS person_name,
-                people.thumbnail_path AS person_thumbnail_path
-            FROM  people
+                p.person_id AS person_id,
+                p.name AS person_name,
+                f.thumbnail_path AS person_thumbnail_path
+            FROM people AS p
+            LEFT OUTER JOIN pictures_faces AS f
+                ON (f.person_id = p.person_id AND f.is_thumbnail = TRUE)
             ORDER BY name ASC",
         )?;
 
@@ -220,7 +234,10 @@ impl Repository {
                 face_id,
                 detected_at,
 
+                is_source_original,
+
                 bounds_path,
+                thumbnail_path,
 
                 bounds_x,
                 bounds_y,
@@ -268,7 +285,10 @@ impl Repository {
                 face_id,
                 detected_at,
 
+                is_source_original,
+
                 bounds_path,
+                thumbnail_path,
 
                 bounds_x,
                 bounds_y,
@@ -335,6 +355,7 @@ impl Repository {
                 SET
                     is_ignored = TRUE,
                     is_confirmed = FALSE,
+                    is_thumbnail = FALSE,
                     person_id = NULL
                 WHERE face_id = ?1",
             )?;
@@ -483,24 +504,34 @@ impl Repository {
         let tx = con.transaction()?;
 
         {
+            // GTK allows the text gtk::Entry input box to be activated multiple times
+            // which results in duplicate people being created :-(
+            // FIXME can we configure gtk::Entry to only be activatible once?
             let mut insert_person = tx.prepare_cached(
                 "
-                WITH face(name, thumbnail_path) AS (
-                    SELECT ?2 as name, thumbnail_path FROM pictures_faces WHERE face_id = ?1
-                )
-                INSERT INTO people (name, thumbnail_path)
-                SELECT name, thumbnail_path FROM face
+                INSERT INTO people (name)
+                SELECT ?1 AS name
+                FROM pictures_faces
+                WHERE face_id = ?2 AND person_id IS NULL
                 ",
             )?;
 
-            insert_person.execute(params![face_id.id(), name])?;
+            insert_person.execute(params![name, face_id.id(),])?;
+
+            // Zero if no rows inserted.
+            // See https://www.sqlite.org/c3ref/last_insert_rowid.html
             let person_id = tx.last_insert_rowid();
+            if person_id == 0 {
+                warn!("Detected double insert of person. Skipping.");
+                return Ok(());
+            }
 
             let mut update_face = tx.prepare_cached(
                 "UPDATE pictures_faces
                 SET
                     person_id = ?2,
-                    is_confirmed = TRUE
+                    is_confirmed = TRUE,
+                    is_thumbnail = TRUE
                 WHERE face_id = ?1",
             )?;
 
@@ -546,7 +577,8 @@ impl Repository {
                 "UPDATE pictures_faces
                 SET
                     person_id = ?2,
-                    is_confirmed = FALSE
+                    is_confirmed = FALSE,
+                    is_thumbnail = FALSE
                 WHERE face_id = ?1",
             )?;
 
@@ -585,7 +617,8 @@ impl Repository {
                 "UPDATE pictures_faces
                 SET
                     person_id = NULL,
-                    is_confirmed = FALSE
+                    is_confirmed = FALSE,
+                    is_thumbnail = FALSE
                 WHERE face_id = ?1",
             )?;
 
@@ -602,12 +635,12 @@ impl Repository {
 
         {
             let mut stmt = tx.prepare_cached(
-                "UPDATE people
+                "UPDATE pictures_faces
                 SET
-                    thumbnail_path = faces.thumbnail_path
-                FROM pictures_faces AS faces
-                WHERE people.person_id = ?1
-                AND faces.face_id = ?2",
+                    is_thumbnail = FALSE
+                WHERE
+                    person_id = ?1
+                    AND face_id != ?2",
             )?;
 
             stmt.execute(params![person_id.id(), face_id.id(),])?;
@@ -615,7 +648,8 @@ impl Repository {
             let mut stmt = tx.prepare_cached(
                 "UPDATE pictures_faces
                 SET
-                    is_confirmed = TRUE
+                    is_confirmed = TRUE,
+                    is_thumbnail = TRUE
                 WHERE face_id = ?1",
             )?;
 
@@ -636,33 +670,38 @@ impl Repository {
             .get("face_thumbnail_path")
             .map(|p: String| self.data_dir_base_path.join(p))?;
 
-        let orientation: Orientation = row
-            .get("orientation")
-            .map(|x: u32| Orientation::from(x))
-            .unwrap_or_default();
-
         let face = model::Face {
             face_id,
             thumbnail_path: face_thumbnail_path,
-            orientation,
         };
 
         let person_id = row.get("person_id").map(PersonId::new).ok();
 
         let person_name = row.get("person_name").ok();
 
-        let person_thumbnail_path = row
-            .get("person_thumbnail_path")
-            .map(|p: String| self.data_dir_base_path.join(p))
-            .ok();
+        let person = if let (Some(person_id), Some(name)) = (person_id, person_name) {
+            let small_thumbnail_path = row
+                .get("person_thumbnail_path")
+                .map(|p: String| self.data_dir_base_path.join(p))
+                .ok()
+                .expect("Must have small thumbnail path");
+
+            // FIXME should this path be in database?
+            let large_thumbnail_path = self
+                .cache_dir_base_path
+                .join("face_thumbnails")
+                .join("large")
+                .join(
+                    small_thumbnail_path
+                        .file_name()
+                        .expect("Must have file name"),
+                );
 
-        let person = if let (Some(person_id), Some(name), Some(thumbnail_path)) =
-            (person_id, person_name, person_thumbnail_path)
-        {
             Some(model::Person {
                 person_id,
                 name,
-                thumbnail_path,
+                small_thumbnail_path: Some(small_thumbnail_path),
+                large_thumbnail_path: Some(large_thumbnail_path),
             })
         } else {
             None
@@ -676,14 +715,28 @@ impl Repository {
 
         let name = row.get("person_name")?;
 
-        let thumbnail_path = row
+        let small_thumbnail_path = row
             .get("person_thumbnail_path")
-            .map(|p: String| self.data_dir_base_path.join(p))?;
+            .map(|p: String| self.data_dir_base_path.join(p))
+            .ok()
+            .expect("Must have small thumbnail path");
+
+        // FIXME should this path be in database?
+        let large_thumbnail_path = self
+            .cache_dir_base_path
+            .join("face_thumbnails")
+            .join("large")
+            .join(
+                small_thumbnail_path
+                    .file_name()
+                    .expect("Must have file name"),
+            );
 
         std::result::Result::Ok(model::Person {
             person_id,
             name,
-            thumbnail_path,
+            small_thumbnail_path: Some(small_thumbnail_path),
+            large_thumbnail_path: Some(large_thumbnail_path),
         })
     }
 
@@ -694,6 +747,10 @@ impl Repository {
             .get("bounds_path")
             .map(|p: String| self.data_dir_base_path.join(p))?;
 
+        let thumbnail_path = row
+            .get("thumbnail_path")
+            .map(|p: String| self.data_dir_base_path.join(p))?;
+
         let bounds = Rect {
             x: row.get("bounds_x")?,
             y: row.get("bounds_y")?,
@@ -720,9 +777,13 @@ impl Repository {
 
         let detected_at = row.get("detected_at")?;
 
+        let is_source_original: bool = row.get("is_source_original")?;
+
         let face = model::DetectedFace {
             face_id,
             face_path,
+            small_thumbnail_path: thumbnail_path,
+            is_source_original,
             bounds,
             right_eye: (right_eye_x, right_eye_y),
             left_eye: (left_eye_x, left_eye_y),
@@ -752,4 +813,89 @@ impl Repository {
 
         std::result::Result::Ok(person)
     }
+
+    pub fn migrate_get_all(&self) -> Result<Vec<FaceToMigrate>> {
+        let con = self.con.lock().unwrap();
+        let mut stmt = con.prepare(
+            "SELECT
+                migrate_faces.face_id AS face_id,
+                migrate_faces.face_index AS face_index,
+                pictures.picture_path_b64 AS picture_path_b64,
+                pictures_faces.bounds_path AS bounds_path,
+                pictures_faces.thumbnail_path AS thumbnail_path
+            FROM migrate_faces
+            INNER JOIN pictures_faces USING (face_id)
+            INNER JOIN pictures USING (picture_id)",
+        )?;
+
+        let result: Vec<model::FaceToMigrate> = stmt
+            .query_map([], |row| self.to_face_to_migrate(row))?
+            .flatten()
+            .collect();
+
+        Ok(result)
+    }
+
+    pub fn migrate_update_face_paths(&mut self, mf: MigratedFace) -> Result<()> {
+        let mut con = self.con.lock().unwrap();
+        let tx = con.transaction()?;
+
+        {
+            let mut stmt = tx.prepare_cached(
+                "UPDATE pictures_faces
+                SET
+                    bounds_path = ?1,
+                    thumbnail_path = ?2
+                WHERE face_id = ?3",
+            )?;
+
+            stmt.execute(params![
+                mf.bounds_path.to_string_lossy(),
+                mf.thumbnail_path.to_string_lossy(),
+                mf.face_id.id(),
+            ])?;
+        }
+
+        tx.commit()?;
+        Ok(())
+    }
+
+    pub fn migrate_truncate(&mut self) -> Result<()> {
+        let mut con = self.con.lock().unwrap();
+        let tx = con.transaction()?;
+        {
+            tx.execute("DELETE FROM migrate_faces", [])?;
+        }
+        tx.commit()?;
+        Ok(())
+    }
+
+    fn to_face_to_migrate(&self, row: &Row<'_>) -> rusqlite::Result<FaceToMigrate> {
+        let face_id = row.get("face_id").map(FaceId::new)?;
+        let face_index: u32 = row.get("face_index")?;
+
+        let picture_relative_path = row
+            .get("picture_path_b64")
+            .ok()
+            .and_then(|x: String| path_encoding::from_base64(&x).ok())
+            .expect("Must have picture path");
+
+        let bounds_path = row
+            .get("bounds_path")
+            .map(|p: String| self.data_dir_base_path.join(p))?;
+
+        let thumbnail_path = row
+            .get("thumbnail_path")
+            .map(|p: String| self.data_dir_base_path.join(p))?;
+
+        let face = model::FaceToMigrate {
+            face_id,
+            face_index,
+            picture_relative_path,
+            bounds_path,
+            thumbnail_path,
+        };
+
+        std::result::Result::Ok(face)
+    }
 }
