@@ -6,6 +6,8 @@ use chrono::TimeDelta;
 use fotema_core::Visual;
 use fotema_core::VisualId;
 use fotema_core::visual::model::PictureOrientation;
+use fotema_core::FlatpakPathBuf;
+
 use glycin;
 use relm4::adw::gdk;
 use relm4::gtk;
@@ -19,7 +21,6 @@ use crate::app::components::progress_monitor::ProgressMonitor;
 use crate::app::components::progress_panel::ProgressPanel;
 use crate::fl;
 
-use std::path::PathBuf;
 use std::sync::Arc;
 
 use tracing::{Level, debug, event, info};
@@ -54,11 +55,8 @@ pub enum Playback {
 
 #[derive(Debug, Eq, PartialEq)]
 pub enum Broken {
-    /// Visual item has no path in database. Shouldn't really happen.
-    MissingPath,
-
     /// Visual item no longer on file system.
-    MissingInFileSystem(PathBuf),
+    MissingInFileSystem(FlatpakPathBuf),
 
     /// Glycin couldn't load the file.
     Failed,
@@ -355,10 +353,7 @@ impl SimpleAsyncComponent for ViewOne {
             ViewOneInput::Load(visual) => {
                 info!("Load visual {}", visual.visual_id);
 
-                let visual_path = visual
-                    .picture_path
-                    .as_ref()
-                    .or_else(|| visual.video_path.as_ref());
+                let visual_sandbox_path = visual.sandbox_path();
 
                 self.viewing = Viewing::None;
                 self.audio = Audio::None;
@@ -367,15 +362,9 @@ impl SimpleAsyncComponent for ViewOne {
                 self.is_skipping_allowed = false;
                 self.visual_id = None;
 
-                let Some(visual_path) = visual_path else {
+                if !visual_sandbox_path.exists() {
                     self.viewing = Viewing::Error;
-                    self.broken = Broken::MissingPath;
-                    return;
-                };
-
-                if !visual_path.exists() {
-                    self.viewing = Viewing::Error;
-                    self.broken = Broken::MissingInFileSystem(visual_path.clone());
+                    self.broken = Broken::MissingInFileSystem(visual.path().clone());
                     return;
                 }
 
@@ -401,7 +390,7 @@ impl SimpleAsyncComponent for ViewOne {
                         .unwrap_or(PictureOrientation::North);
                     self.picture.add_css_class(orientation.as_ref());
 
-                    let file = gio::File::for_path(visual_path);
+                    let file = gio::File::for_path(visual_sandbox_path);
 
                     let mut loader = glycin::Loader::new(file);
                     loader.apply_transformations(false);
@@ -450,16 +439,18 @@ impl SimpleAsyncComponent for ViewOne {
                         }
 
                         let video_path = visual
-                            .video_transcoded_path
-                            .as_ref()
+                            .video_transcoded_path.clone()
                             .filter(|x| x.exists())
-                            .or_else(|| visual.video_path.as_ref())
+                            .or_else(|| visual.video_path.clone().map(|p| p.sandbox_path))
                             .filter(|x| x.exists())
-                            .or_else(|| visual.motion_photo_video_path.as_ref())
+                            .or_else(|| visual.motion_photo_video_path.clone())
                             .expect("must have video path");
 
+                        debug!("Video path is: {:?}", video_path);
+
                         let video = gtk::MediaFile::for_filename(video_path);
                         if visual.is_motion_photo() {
+                            debug!("Is a motion photo");
                             self.viewing = Viewing::MotionPhoto;
 
                             self.playback = Playback::Playing;
@@ -669,7 +660,6 @@ impl ViewOne {
 
     fn broken_status_icon_name(&self) -> Option<&str> {
         match self.broken {
-            Broken::MissingPath => Some("item-missing-symbolic"),
             Broken::MissingInFileSystem(_) => Some("item-missing-symbolic"),
             Broken::Failed => Some("sad-computer-symbolic"),
             Broken::None => None,
@@ -678,10 +668,9 @@ impl ViewOne {
 
     fn broken_status_description(&self) -> Option<String> {
         match self.broken {
-            Broken::MissingPath => Some(fl!("viewer-error-missing-path")),
             Broken::MissingInFileSystem(ref visual_path) => Some(fl!(
                 "viewer-error-missing-file",
-                file_name = visual_path.to_string_lossy()
+                file_name = visual_path.host_path.to_string_lossy()
             )),
             Broken::Failed => Some(fl!("viewer-error-failed-to-load")),
             Broken::None => None::<String>,
